@@ -3,8 +3,9 @@ use std::fmt::{Display, Formatter};
 use pep508_rs::PackageName;
 
 use crate::{
-    Dist, DistributionId, DistributionMetadata, Identifier, IndexUrl, InstalledDist, Name,
-    ResourceId, VersionOrUrlRef,
+    BuiltDist, Dist, DistributionId, DistributionMetadata, Identifier, IndexUrl, InstalledDist,
+    Name, PrioritizedDist, RegistryBuiltWheel, RegistrySourceDist, ResourceId, SourceDist,
+    VersionOrUrlRef,
 };
 
 /// A distribution that can be used for resolution and installation.
@@ -20,7 +21,18 @@ pub enum ResolvedDist {
 #[derive(Debug, Clone)]
 pub enum ResolvedDistRef<'a> {
     Installed(&'a InstalledDist),
-    Installable(&'a Dist),
+    InstallableRegistrySourceDist {
+        /// The source distribution that should be used.
+        sdist: &'a RegistrySourceDist,
+        /// The prioritized distribution that the wheel came from.
+        prioritized: &'a PrioritizedDist,
+    },
+    InstallableRegistryBuiltDist {
+        /// The wheel that should be used.
+        wheel: &'a RegistryBuiltWheel,
+        /// The prioritized distribution that the wheel came from.
+        prioritized: &'a PrioritizedDist,
+    },
 }
 
 impl ResolvedDist {
@@ -44,7 +56,30 @@ impl ResolvedDist {
 impl ResolvedDistRef<'_> {
     pub fn to_owned(&self) -> ResolvedDist {
         match self {
-            Self::Installable(dist) => ResolvedDist::Installable((*dist).clone()),
+            Self::InstallableRegistrySourceDist { sdist, prioritized } => {
+                // This is okay because we're only here if the prioritized dist
+                // has an sdist, so this always succeeds.
+                let source = prioritized.source_dist().expect("a source distribution");
+                assert_eq!(
+                    (&sdist.name, &sdist.version),
+                    (&source.name, &source.version),
+                    "expected chosen sdist to match prioritized sdist"
+                );
+                ResolvedDist::Installable(Dist::Source(SourceDist::Registry(source)))
+            }
+            Self::InstallableRegistryBuiltDist {
+                wheel, prioritized, ..
+            } => {
+                assert_eq!(
+                    Some(&wheel.filename),
+                    prioritized.best_wheel().map(|(wheel, _)| &wheel.filename),
+                    "expected chosen wheel to match best wheel"
+                );
+                // This is okay because we're only here if the prioritized dist
+                // has at least one wheel, so this always succeeds.
+                let built = prioritized.built_dist().expect("at least one wheel");
+                ResolvedDist::Installable(Dist::Built(BuiltDist::Registry(built)))
+            }
             Self::Installed(dist) => ResolvedDist::Installed((*dist).clone()),
         }
     }
@@ -53,7 +88,8 @@ impl ResolvedDistRef<'_> {
 impl Display for ResolvedDistRef<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Installable(dist) => Display::fmt(dist, f),
+            Self::InstallableRegistrySourceDist { sdist, .. } => Display::fmt(sdist, f),
+            Self::InstallableRegistryBuiltDist { wheel, .. } => Display::fmt(wheel, f),
             Self::Installed(dist) => Display::fmt(dist, f),
         }
     }
@@ -62,7 +98,8 @@ impl Display for ResolvedDistRef<'_> {
 impl Name for ResolvedDistRef<'_> {
     fn name(&self) -> &PackageName {
         match self {
-            Self::Installable(dist) => dist.name(),
+            Self::InstallableRegistrySourceDist { sdist, .. } => sdist.name(),
+            Self::InstallableRegistryBuiltDist { wheel, .. } => wheel.name(),
             Self::Installed(dist) => dist.name(),
         }
     }
@@ -72,7 +109,8 @@ impl DistributionMetadata for ResolvedDistRef<'_> {
     fn version_or_url(&self) -> VersionOrUrlRef {
         match self {
             Self::Installed(installed) => VersionOrUrlRef::Version(installed.version()),
-            Self::Installable(dist) => dist.version_or_url(),
+            Self::InstallableRegistrySourceDist { sdist, .. } => sdist.version_or_url(),
+            Self::InstallableRegistryBuiltDist { wheel, .. } => wheel.version_or_url(),
         }
     }
 }
@@ -81,14 +119,16 @@ impl Identifier for ResolvedDistRef<'_> {
     fn distribution_id(&self) -> DistributionId {
         match self {
             Self::Installed(dist) => dist.distribution_id(),
-            Self::Installable(dist) => dist.distribution_id(),
+            Self::InstallableRegistrySourceDist { sdist, .. } => sdist.distribution_id(),
+            Self::InstallableRegistryBuiltDist { wheel, .. } => wheel.distribution_id(),
         }
     }
 
     fn resource_id(&self) -> ResourceId {
         match self {
             Self::Installed(dist) => dist.resource_id(),
-            Self::Installable(dist) => dist.resource_id(),
+            Self::InstallableRegistrySourceDist { sdist, .. } => sdist.resource_id(),
+            Self::InstallableRegistryBuiltDist { wheel, .. } => wheel.resource_id(),
         }
     }
 }
