@@ -11,7 +11,7 @@ use zip::result::ZipError;
 use pep440_rs::Version;
 use platform_tags::{Arch, Os};
 use pypi_types::Scheme;
-pub use uninstall::{uninstall_wheel, Uninstall};
+pub use uninstall::{uninstall_egg, uninstall_legacy_editable, uninstall_wheel, Uninstall};
 use uv_fs::Simplified;
 use uv_normalize::PackageName;
 
@@ -82,8 +82,10 @@ pub enum Error {
     DirectUrlJson(#[from] serde_json::Error),
     #[error("No .dist-info directory found")]
     MissingDistInfo,
-    #[error("Cannot uninstall package; RECORD file not found at: {}", _0.user_display())]
+    #[error("Cannot uninstall package; `RECORD` file not found at: {}", _0.user_display())]
     MissingRecord(PathBuf),
+    #[error("Cannot uninstall package; `top_level.txt` file not found at: {}", _0.user_display())]
+    MissingTopLevel(PathBuf),
     #[error("Multiple .dist-info directories found: {0}")]
     MultipleDistInfo(String),
     #[error(
@@ -106,4 +108,6 @@ pub enum Error {
     MismatchedName(PackageName, PackageName),
     #[error("Wheel version does not match filename: {0} != {1}")]
     MismatchedVersion(Version, Version),
+    #[error("Invalid egg-link")]
+    InvalidEggLink(PathBuf),
 }
