@@ -10,8 +10,8 @@ use uv_normalize::ExtraName;
 use crate::marker::parse_markers_cursor;
 use crate::{
     expand_env_vars, parse_extras_cursor, split_extras, split_scheme, strip_host, Cursor,
-    MarkerEnvironment, MarkerTree, Pep508Error, Pep508ErrorSource, Scheme, VerbatimUrl,
-    VerbatimUrlError,
+    MarkerEnvironment, MarkerTree, Pep508Error, Pep508ErrorSource, Reporter, RequirementOrigin,
+    Scheme, TracingReporter, VerbatimUrl, VerbatimUrlError,
 };
 
 /// A PEP 508-like, direct URL dependency specifier without a package name.
@@ -30,17 +30,37 @@ pub struct UnnamedRequirement {
     /// `requests [security,tests] >= 2.8.1, == 2.8.* ; python_version > "3.8"`.
     /// Those are a nested and/or tree.
     pub marker: Option<MarkerTree>,
+    /// The source file containing the requirement.
+    pub origin: Option<RequirementOrigin>,
 }
 
 impl UnnamedRequirement {
     /// Returns whether the markers apply for the given environment
     pub fn evaluate_markers(&self, env: &MarkerEnvironment, extras: &[ExtraName]) -> bool {
+        self.evaluate_optional_environment(Some(env), extras)
+    }
+
+    /// Returns whether the markers apply for the given environment
+    pub fn evaluate_optional_environment(
+        &self,
+        env: Option<&MarkerEnvironment>,
+        extras: &[ExtraName],
+    ) -> bool {
         if let Some(marker) = &self.marker {
-            marker.evaluate(env, extras)
+            marker.evaluate_optional_environment(env, extras)
         } else {
             true
         }
     }
+
+    /// Set the source file containing the requirement.
+    #[must_use]
+    pub fn with_origin(self, origin: RequirementOrigin) -> Self {
+        Self {
+            origin: Some(origin),
+            ..self
+        }
+    }
 }
 
 impl Display for UnnamedRequirement {
@@ -90,7 +110,7 @@ impl FromStr for UnnamedRequirement {
 
     /// Parse a PEP 508-like direct URL requirement without a package name.
     fn from_str(input: &str) -> Result<Self, Self::Err> {
-        parse_unnamed_requirement(&mut Cursor::new(input), None)
+        parse_unnamed_requirement(&mut Cursor::new(input), None, &mut TracingReporter)
     }
 }
 
@@ -99,8 +119,13 @@ impl UnnamedRequirement {
     pub fn parse(
         input: &str,
         working_dir: impl AsRef<Path>,
+        reporter: &mut impl Reporter,
     ) -> Result<Self, Pep508Error<VerbatimUrl>> {
-        parse_unnamed_requirement(&mut Cursor::new(input), Some(working_dir.as_ref()))
+        parse_unnamed_requirement(
+            &mut Cursor::new(input),
+            Some(working_dir.as_ref()),
+            reporter,
+        )
     }
 }
 
@@ -110,6 +135,7 @@ impl UnnamedRequirement {
 fn parse_unnamed_requirement(
     cursor: &mut Cursor,
     working_dir: Option<&Path>,
+    reporter: &mut impl Reporter,
 ) -> Result<UnnamedRequirement, Pep508Error<VerbatimUrl>> {
     cursor.eat_whitespace();
 
@@ -123,7 +149,7 @@ fn parse_unnamed_requirement(
     let marker = if cursor.peek_char() == Some(';') {
         // Skip past the semicolon
         cursor.next();
-        Some(parse_markers_cursor(cursor)?)
+        Some(parse_markers_cursor(cursor, reporter)?)
     } else {
         None
     };
@@ -159,6 +185,7 @@ fn parse_unnamed_requirement(
         url,
         extras,
         marker,
+        origin: None,
     })
 }
 
@@ -209,6 +236,12 @@ fn preprocess_unnamed_url(
                 #[cfg(feature = "non-pep508-extensions")]
                 if let Some(working_dir) = working_dir {
                     let url = VerbatimUrl::parse_path(path.as_ref(), working_dir)
+                        .map_err(|err| Pep508Error {
+                            message: Pep508ErrorSource::<VerbatimUrl>::UrlError(err),
+                            start,
+                            len,
+                            input: cursor.to_string(),
+                        })?
                         .with_given(url.to_string());
                     return Ok((url, extras));
                 }
@@ -243,6 +276,12 @@ fn preprocess_unnamed_url(
             _ => {
                 if let Some(working_dir) = working_dir {
                     let url = VerbatimUrl::parse_path(expanded.as_ref(), working_dir)
+                        .map_err(|err| Pep508Error {
+                            message: Pep508ErrorSource::<VerbatimUrl>::UrlError(err),
+                            start,
+                            len,
+                            input: cursor.to_string(),
+                        })?
                         .with_given(url.to_string());
                     return Ok((url, extras));
                 }
@@ -261,8 +300,14 @@ fn preprocess_unnamed_url(
     } else {
         // Ex) `../editable/`
         if let Some(working_dir) = working_dir {
-            let url =
-                VerbatimUrl::parse_path(expanded.as_ref(), working_dir).with_given(url.to_string());
+            let url = VerbatimUrl::parse_path(expanded.as_ref(), working_dir)
+                .map_err(|err| Pep508Error {
+                    message: Pep508ErrorSource::<VerbatimUrl>::UrlError(err),
+                    start,
+                    len,
+                    input: cursor.to_string(),
+                })?
+                .with_given(url.to_string());
             return Ok((url, extras));
         }
 
