@@ -117,7 +117,7 @@ This is a little extra cost, but the idea is that a `CachePolicy` (not an
 `ArchivedCachePolicy`) should only be used in the slower path (i.e., when you
 actually need to make an HTTP request).
 
-[`rkyv::vec::ArchivedVec`]: hhttps://docs.rs/rkyv/0.7.43/rkyv/vec/struct.ArchivedVec.html
+[`rkyv::vec::ArchivedVec`]: https://docs.rs/rkyv/0.7.43/rkyv/vec/struct.ArchivedVec.html
 [`rkyv::string::ArchivedString`]: https://docs.rs/rkyv/0.7.43/rkyv/string/struct.ArchivedString.html
 
 # Additional reading
@@ -422,6 +422,10 @@ impl ArchivedCachePolicy {
         //
         // [RFC 9111 S4.3.3]: https://www.rfc-editor.org/rfc/rfc9111.html#section-4.3.3
         if new_policy.response.status != 304 {
+            tracing::trace!(
+                "is modified because status is {:?} and not 304",
+                new_policy.response.status
+            );
             return true;
         }
         // As per [RFC 9111 S4.3.4], we need to confirm that our validators match. Here,
@@ -433,6 +437,10 @@ impl ArchivedCachePolicy {
                 // We don't support weak validators, so only match if they're
                 // both strong.
                 if !old_etag.weak && !new_etag.weak && old_etag.value == new_etag.value {
+                    tracing::trace!(
+                        "not modified because old and new etag values ({:?}) match",
+                        new_etag.value,
+                    );
                     return false;
                 }
             }
@@ -450,6 +458,9 @@ impl ArchivedCachePolicy {
                 .as_ref()
             {
                 if old_last_modified == new_last_modified {
+                    tracing::trace!(
+                        "not modified because modified times ({new_last_modified:?}) match",
+                    );
                     return false;
                 }
             }
@@ -468,6 +479,10 @@ impl ArchivedCachePolicy {
                 .last_modified_unix_timestamp
                 .is_none()
         {
+            tracing::trace!(
+                "not modified because there are no etags or last modified \
+                 timestamps, so we assume the 304 status is correct",
+            );
             return false;
         }
         true
