@@ -22,6 +22,8 @@ pub enum RequirementsSource {
     SetupPy(PathBuf),
     /// Dependencies were provided via a `setup.cfg` file (e.g., `pip-compile setup.cfg`).
     SetupCfg(PathBuf),
+    /// Dependencies were provided via a path to a source tree (e.g., `pip install .`).
+    SourceTree(PathBuf),
 }
 
 impl RequirementsSource {
@@ -122,6 +124,12 @@ impl RequirementsSource {
         Self::Package(name)
     }
 
+    /// Parse a [`RequirementsSource`] from a user-provided string, assumed to be a path to a source
+    /// tree.
+    pub fn from_source_tree(path: PathBuf) -> Self {
+        Self::SourceTree(path)
+    }
+
     /// Returns `true` if the source allows extras to be specified.
     pub fn allows_extras(&self) -> bool {
         matches!(
@@ -139,7 +147,8 @@ impl std::fmt::Display for RequirementsSource {
             Self::RequirementsTxt(path)
             | Self::PyprojectToml(path)
             | Self::SetupPy(path)
-            | Self::SetupCfg(path) => {
+            | Self::SetupCfg(path)
+            | Self::SourceTree(path) => {
                 write!(f, "{}", path.simplified_display())
             }
         }
