@@ -1,17 +1,22 @@
-use std::path::PathBuf;
+use std::collections::BTreeMap;
+use std::path::{Path, PathBuf};
 
 use anyhow::{Context, Result};
+use itertools::{Either, Itertools};
 use rustc_hash::FxHashSet;
-use tracing::instrument;
+use tracing::{debug, instrument};
 
 use cache_key::CanonicalUrl;
-use distribution_types::{FlatIndexLocation, IndexUrl};
-use pep508_rs::Requirement;
+use distribution_types::{
+    FlatIndexLocation, IndexUrl, Requirement, RequirementSource, UnresolvedRequirement,
+    UnresolvedRequirementSpecification,
+};
+use pep508_rs::{UnnamedRequirement, VerbatimUrl};
 use requirements_txt::{
     EditableRequirement, FindLink, RequirementEntry, RequirementsTxt, RequirementsTxtRequirement,
 };
 use uv_client::BaseClientBuilder;
-use uv_configuration::{NoBinary, NoBuild};
+use uv_configuration::{NoBinary, NoBuild, PreviewMode};
 use uv_fs::Simplified;
 use uv_normalize::{ExtraName, PackageName};
 
@@ -23,11 +28,11 @@ pub struct RequirementsSpecification {
     /// The name of the project specifying requirements.
     pub project: Option<PackageName>,
     /// The requirements for the project.
-    pub requirements: Vec<RequirementEntry>,
+    pub requirements: Vec<UnresolvedRequirementSpecification>,
     /// The constraints for the project.
     pub constraints: Vec<Requirement>,
     /// The overrides for the project.
-    pub overrides: Vec<RequirementEntry>,
+    pub overrides: Vec<UnresolvedRequirementSpecification>,
     /// Package to install as editable installs
     pub editables: Vec<EditableRequirement>,
     /// The source trees from which to extract requirements.
@@ -55,60 +60,45 @@ impl RequirementsSpecification {
         source: &RequirementsSource,
         extras: &ExtrasSpecification,
         client_builder: &BaseClientBuilder<'_>,
+        preview: PreviewMode,
     ) -> Result<Self> {
         Ok(match source {
             RequirementsSource::Package(name) => {
                 let requirement = RequirementsTxtRequirement::parse(name, std::env::current_dir()?)
                     .with_context(|| format!("Failed to parse `{name}`"))?;
                 Self {
-                    project: None,
-                    requirements: vec![RequirementEntry {
-                        requirement,
-                        hashes: vec![],
-                    }],
-                    constraints: vec![],
-                    overrides: vec![],
-                    editables: vec![],
-                    source_trees: vec![],
-                    extras: FxHashSet::default(),
-                    index_url: None,
-                    extra_index_urls: vec![],
-                    no_index: false,
-                    find_links: vec![],
-                    no_binary: NoBinary::default(),
-                    no_build: NoBuild::default(),
+                    requirements: vec![UnresolvedRequirementSpecification::try_from(
+                        RequirementEntry {
+                            requirement,
+                            hashes: vec![],
+                        },
+                    )?],
+                    ..Self::default()
                 }
             }
             RequirementsSource::Editable(name) => {
-                let requirement = EditableRequirement::parse(name, std::env::current_dir()?)
+                let requirement = EditableRequirement::parse(name, None, std::env::current_dir()?)
                     .with_context(|| format!("Failed to parse `{name}`"))?;
                 Self {
-                    project: None,
-                    requirements: vec![],
-                    constraints: vec![],
-                    overrides: vec![],
                     editables: vec![requirement],
-                    source_trees: vec![],
-                    extras: FxHashSet::default(),
-                    index_url: None,
-                    extra_index_urls: vec![],
-                    no_index: false,
-                    find_links: vec![],
-                    no_binary: NoBinary::default(),
-                    no_build: NoBuild::default(),
+                    ..Self::default()
                 }
             }
             RequirementsSource::RequirementsTxt(path) => {
                 let requirements_txt =
                     RequirementsTxt::parse(path, std::env::current_dir()?, client_builder).await?;
                 Self {
-                    project: None,
-                    requirements: requirements_txt.requirements,
-                    constraints: requirements_txt.constraints,
-                    overrides: vec![],
+                    requirements: requirements_txt
+                        .requirements
+                        .into_iter()
+                        .map(UnresolvedRequirementSpecification::try_from)
+                        .collect::<Result<_, _>>()?,
+                    constraints: requirements_txt
+                        .constraints
+                        .into_iter()
+                        .map(Requirement::from_pep508)
+                        .collect::<Result<_, _>>()?,
                     editables: requirements_txt.editables,
-                    source_trees: vec![],
-                    extras: FxHashSet::default(),
                     index_url: requirements_txt.index_url.map(IndexUrl::from),
                     extra_index_urls: requirements_txt
                         .extra_index_urls
@@ -126,99 +116,115 @@ impl RequirementsSpecification {
                         .collect(),
                     no_binary: requirements_txt.no_binary,
                     no_build: requirements_txt.only_binary,
+                    ..Self::default()
                 }
             }
             RequirementsSource::PyprojectToml(path) => {
-                let contents = uv_fs::read_to_string(path).await?;
-                let pyproject = toml::from_str::<PyProjectToml>(&contents)
-                    .with_context(|| format!("Failed to parse `{}`", path.user_display()))?;
+                let contents = uv_fs::read_to_string(&path).await?;
+                Self::parse_direct_pyproject_toml(&contents, extras, path.as_ref(), preview)
+                    .with_context(|| format!("Failed to parse `{}`", path.user_display()))?
+            }
+            RequirementsSource::SetupPy(path) | RequirementsSource::SetupCfg(path) => Self {
+                source_trees: vec![path.clone()],
+                ..Self::default()
+            },
+            RequirementsSource::SourceTree(path) => Self {
+                project: None,
+                requirements: vec![UnresolvedRequirementSpecification {
+                    requirement: UnresolvedRequirement::Unnamed(UnnamedRequirement {
+                        url: VerbatimUrl::from_path(path)?,
+                        extras: vec![],
+                        marker: None,
+                        origin: None,
+                    }),
+                    hashes: vec![],
+                }],
+                ..Self::default()
+            },
+        })
+    }
 
-                // Attempt to read metadata from the `pyproject.toml` directly.
-                //
-                // If we fail to extract the PEP 621 metadata, fall back to treating it as a source
-                // tree, as there are some cases where the `pyproject.toml` may not be a valid PEP
-                // 621 file, but might still resolve under PEP 517. (If the source tree doesn't
-                // resolve under PEP 517, we'll catch that later.)
-                //
-                // For example, Hatch's "Context formatting" API is not compliant with PEP 621, as
-                // it expects dynamic processing by the build backend for the static metadata
-                // fields. See: https://hatch.pypa.io/latest/config/context/
-                if let Some(project) = pyproject
-                    .project
-                    .and_then(|project| Pep621Metadata::try_from(project, extras).ok().flatten())
-                {
-                    Self {
-                        project: Some(project.name),
-                        requirements: project
-                            .requirements
-                            .into_iter()
-                            .map(|requirement| RequirementEntry {
-                                requirement: RequirementsTxtRequirement::Pep508(requirement),
+    /// Attempt to read metadata from the `pyproject.toml` directly.
+    ///
+    /// Since we only use this path for directly included pyproject.toml, we are strict about
+    /// PEP 621 and don't allow invalid `project.dependencies` (e.g., Hatch's relative path
+    /// support).
+    pub(crate) fn parse_direct_pyproject_toml(
+        contents: &str,
+        extras: &ExtrasSpecification,
+        pyproject_path: &Path,
+        preview: PreviewMode,
+    ) -> Result<Self> {
+        let pyproject = toml::from_str::<PyProjectToml>(contents)?;
+
+        // We need use this path as base for the relative paths inside pyproject.toml, so
+        // we need the absolute path instead of a potentially relative path. E.g. with
+        // `foo = { path = "../foo" }`, we will join `../foo` onto this path.
+        let absolute_path = uv_fs::absolutize_path(pyproject_path)?;
+        let project_dir = absolute_path
+            .parent()
+            .context("`pyproject.toml` has no parent directory")?;
+
+        let workspace_sources = BTreeMap::default();
+        let workspace_packages = BTreeMap::default();
+        match Pep621Metadata::try_from(
+            pyproject,
+            extras,
+            pyproject_path,
+            project_dir,
+            &workspace_sources,
+            &workspace_packages,
+            preview,
+        ) {
+            Ok(Some(project)) => {
+                // Partition into editable and non-editable requirements.
+                let (editables, requirements): (Vec<_>, Vec<_>) = project
+                    .requirements
+                    .into_iter()
+                    .partition_map(|requirement| {
+                        if let RequirementSource::Path {
+                            path,
+                            editable: true,
+                            url,
+                        } = requirement.source
+                        {
+                            Either::Left(EditableRequirement {
+                                url,
+                                path,
+                                marker: requirement.marker,
+                                extras: requirement.extras,
+                                origin: requirement.origin,
+                            })
+                        } else {
+                            Either::Right(UnresolvedRequirementSpecification {
+                                requirement: UnresolvedRequirement::Named(requirement),
                                 hashes: vec![],
                             })
-                            .collect(),
-                        constraints: vec![],
-                        overrides: vec![],
-                        editables: vec![],
-                        source_trees: vec![],
-                        extras: project.used_extras,
-                        index_url: None,
-                        extra_index_urls: vec![],
-                        no_index: false,
-                        find_links: vec![],
-                        no_binary: NoBinary::default(),
-                        no_build: NoBuild::default(),
-                    }
-                } else {
-                    let path = fs_err::canonicalize(path)?;
-                    let source_tree = path.parent().ok_or_else(|| {
-                        anyhow::anyhow!(
-                            "The file `{}` appears to be a `pyproject.toml` file, which must be in a directory",
-                            path.user_display()
-                        )
-                    })?;
-                    Self {
-                        project: None,
-                        requirements: vec![],
-                        constraints: vec![],
-                        overrides: vec![],
-                        editables: vec![],
-                        source_trees: vec![source_tree.to_path_buf()],
-                        extras: FxHashSet::default(),
-                        index_url: None,
-                        extra_index_urls: vec![],
-                        no_index: false,
-                        find_links: vec![],
-                        no_binary: NoBinary::default(),
-                        no_build: NoBuild::default(),
-                    }
-                }
+                        }
+                    });
+
+                Ok(Self {
+                    project: Some(project.name),
+                    editables,
+                    requirements,
+                    extras: project.used_extras,
+                    ..Self::default()
+                })
             }
-            RequirementsSource::SetupPy(path) | RequirementsSource::SetupCfg(path) => {
-                let path = fs_err::canonicalize(path)?;
-                let source_tree = path.parent().ok_or_else(|| {
-                    anyhow::anyhow!(
-                        "The file `{}` appears to be a `setup.py` or `setup.cfg` file, which must be in a directory",
-                        path.user_display()
-                    )
-                })?;
-                Self {
+            Ok(None) => {
+                debug!(
+                    "Dynamic pyproject.toml at: `{}`",
+                    pyproject_path.user_display()
+                );
+                Ok(Self {
                     project: None,
                     requirements: vec![],
-                    constraints: vec![],
-                    overrides: vec![],
-                    editables: vec![],
-                    source_trees: vec![source_tree.to_path_buf()],
-                    extras: FxHashSet::default(),
-                    index_url: None,
-                    extra_index_urls: vec![],
-                    no_index: false,
-                    find_links: vec![],
-                    no_binary: NoBinary::default(),
-                    no_build: NoBuild::default(),
-                }
+                    source_trees: vec![pyproject_path.to_path_buf()],
+                    ..Self::default()
+                })
             }
-        })
+            Err(err) => Err(err.into()),
+        }
     }
 
     /// Read the combined requirements and constraints from a set of sources.
@@ -228,6 +234,7 @@ impl RequirementsSpecification {
         overrides: &[RequirementsSource],
         extras: &ExtrasSpecification,
         client_builder: &BaseClientBuilder<'_>,
+        preview: PreviewMode,
     ) -> Result<Self> {
         let mut spec = Self::default();
 
@@ -235,7 +242,7 @@ impl RequirementsSpecification {
         // A `requirements.txt` can contain a `-c constraints.txt` directive within it, so reading
         // a requirements file can also add constraints.
         for source in requirements {
-            let source = Self::from_source(source, extras, client_builder).await?;
+            let source = Self::from_source(source, extras, client_builder, preview).await?;
             spec.requirements.extend(source.requirements);
             spec.constraints.extend(source.constraints);
             spec.overrides.extend(source.overrides);
@@ -268,13 +275,13 @@ impl RequirementsSpecification {
         // Read all constraints, treating both requirements _and_ constraints as constraints.
         // Overrides are ignored, as are the hashes, as they are not relevant for constraints.
         for source in constraints {
-            let source = Self::from_source(source, extras, client_builder).await?;
+            let source = Self::from_source(source, extras, client_builder, preview).await?;
             for entry in source.requirements {
                 match entry.requirement {
-                    RequirementsTxtRequirement::Pep508(requirement) => {
+                    UnresolvedRequirement::Named(requirement) => {
                         spec.constraints.push(requirement);
                     }
-                    RequirementsTxtRequirement::Unnamed(requirement) => {
+                    UnresolvedRequirement::Unnamed(requirement) => {
                         return Err(anyhow::anyhow!(
                             "Unnamed requirements are not allowed as constraints (found: `{requirement}`)"
                         ));
@@ -303,7 +310,7 @@ impl RequirementsSpecification {
         // Read all overrides, treating both requirements _and_ overrides as overrides.
         // Constraints are ignored.
         for source in overrides {
-            let source = Self::from_source(source, extras, client_builder).await?;
+            let source = Self::from_source(source, extras, client_builder, preview).await?;
             spec.overrides.extend(source.requirements);
             spec.overrides.extend(source.overrides);
 
@@ -331,6 +338,7 @@ impl RequirementsSpecification {
     pub async fn from_simple_sources(
         requirements: &[RequirementsSource],
         client_builder: &BaseClientBuilder<'_>,
+        preview: PreviewMode,
     ) -> Result<Self> {
         Self::from_sources(
             requirements,
@@ -338,6 +346,7 @@ impl RequirementsSpecification {
             &[],
             &ExtrasSpecification::None,
             client_builder,
+            preview,
         )
         .await
     }
