@@ -1,4 +1,5 @@
 pub use dependency_mode::DependencyMode;
+pub use editables::BuiltEditableMetadata;
 pub use error::ResolveError;
 pub use exclude_newer::ExcludeNewer;
 pub use exclusions::Exclusions;
@@ -31,6 +32,7 @@ mod exclusions;
 mod flat_index;
 mod lock;
 mod manifest;
+mod marker;
 mod options;
 mod pins;
 mod preferences;
