@@ -1,25 +1,26 @@
 use std::collections::btree_map::{BTreeMap, Entry};
 use std::sync::OnceLock;
 
-use chrono::{DateTime, Utc};
+use rkyv::{de::deserializers::SharedDeserializeMap, Deserialize};
 use rustc_hash::FxHashSet;
-use tracing::{instrument, warn};
+use tracing::instrument;
 
 use distribution_filename::{DistFilename, WheelFilename};
 use distribution_types::{
-    Dist, IncompatibleSource, IncompatibleWheel, IndexUrl, PrioritizedDist,
-    SourceDistCompatibility, WheelCompatibility,
+    HashComparison, IncompatibleSource, IncompatibleWheel, IndexUrl, PrioritizedDist,
+    RegistryBuiltWheel, RegistrySourceDist, SourceDistCompatibility, WheelCompatibility,
 };
 use pep440_rs::{Version, VersionSpecifiers};
-use platform_tags::Tags;
-use pypi_types::{Hashes, Yanked};
-use rkyv::{de::deserializers::SharedDeserializeMap, Deserialize};
-use uv_client::{FlatDistributions, OwnedArchive, SimpleMetadata, VersionFiles};
+use platform_tags::{TagCompatibility, Tags};
+use pypi_types::{HashDigest, Yanked};
+use uv_client::{OwnedArchive, SimpleMetadata, VersionFiles};
+use uv_configuration::{NoBinary, NoBuild};
 use uv_normalize::PackageName;
-use uv_types::{NoBinary, NoBuild};
+use uv_types::HashStrategy;
 use uv_warnings::warn_user_once;
 
-use crate::{python_requirement::PythonRequirement, yanks::AllowedYanks};
+use crate::flat_index::FlatDistributions;
+use crate::{python_requirement::PythonRequirement, yanks::AllowedYanks, ExcludeNewer};
 
 /// A map from versions to distributions.
 #[derive(Debug)]
@@ -46,7 +47,8 @@ impl VersionMap {
         tags: &Tags,
         python_requirement: &PythonRequirement,
         allowed_yanks: &AllowedYanks,
-        exclude_newer: Option<&DateTime<Utc>>,
+        hasher: &HashStrategy,
+        exclude_newer: Option<&ExcludeNewer>,
         flat_index: Option<FlatDistributions>,
         no_binary: &NoBinary,
         no_build: &NoBuild,
@@ -109,6 +111,7 @@ impl VersionMap {
             .allowed_versions(package_name)
             .cloned()
             .unwrap_or_default();
+        let required_hashes = hasher.get_package(package_name).digests().to_vec();
         Self {
             inner: VersionMapInner::Lazy(VersionMapLazy {
                 map,
@@ -120,6 +123,7 @@ impl VersionMap {
                 python_requirement: python_requirement.clone(),
                 exclude_newer: exclude_newer.copied(),
                 allowed_yanks,
+                required_hashes,
             }),
         }
     }
@@ -176,7 +180,7 @@ impl VersionMap {
     }
 
     /// Return the [`Hashes`] for the given version, if any.
-    pub(crate) fn hashes(&self, version: &Version) -> Option<Vec<Hashes>> {
+    pub(crate) fn hashes(&self, version: &Version) -> Option<Vec<HashDigest>> {
         match self.inner {
             VersionMapInner::Eager(ref map) => map.get(version).map(|file| file.hashes().to_vec()),
             VersionMapInner::Lazy(ref lazy) => lazy.get(version).map(|file| file.hashes().to_vec()),
@@ -299,9 +303,11 @@ struct VersionMapLazy {
     /// exists) is satisfied or not.
     python_requirement: PythonRequirement,
     /// Whether files newer than this timestamp should be excluded or not.
-    exclude_newer: Option<DateTime<Utc>>,
+    exclude_newer: Option<ExcludeNewer>,
     /// Which yanked versions are allowed
     allowed_yanks: FxHashSet<Version>,
+    /// The hashes of allowed distributions.
+    required_hashes: Vec<HashDigest>,
 }
 
 impl VersionMapLazy {
@@ -378,38 +384,42 @@ impl VersionMapLazy {
                 let version = filename.version().clone();
                 let requires_python = file.requires_python.clone();
                 let yanked = file.yanked.clone();
-                let hash = file.hashes.clone();
+                let hashes = file.hashes.clone();
                 match filename {
                     DistFilename::WheelFilename(filename) => {
                         let compatibility = self.wheel_compatibility(
                             &filename,
                             &version,
                             requires_python,
+                            &hashes,
                             yanked,
                             excluded,
                             upload_time,
                         );
-                        let dist = Dist::from_registry(
-                            DistFilename::WheelFilename(filename),
-                            file,
-                            self.index.clone(),
-                        );
-                        priority_dist.insert_built(dist, Some(hash), compatibility);
+                        let dist = RegistryBuiltWheel {
+                            filename,
+                            file: Box::new(file),
+                            index: self.index.clone(),
+                        };
+                        priority_dist.insert_built(dist, hashes, compatibility);
                     }
                     DistFilename::SourceDistFilename(filename) => {
                         let compatibility = self.source_dist_compatibility(
                             &version,
                             requires_python,
+                            &hashes,
                             yanked,
                             excluded,
                             upload_time,
                         );
-                        let dist = Dist::from_registry(
-                            DistFilename::SourceDistFilename(filename),
-                            file,
-                            self.index.clone(),
-                        );
-                        priority_dist.insert_source(dist, Some(hash), compatibility);
+                        let dist = RegistrySourceDist {
+                            name: filename.name.clone(),
+                            version: filename.version.clone(),
+                            file: Box::new(file),
+                            index: self.index.clone(),
+                            wheels: vec![],
+                        };
+                        priority_dist.insert_source(dist, hashes, compatibility);
                     }
                 }
             }
@@ -422,10 +432,12 @@ impl VersionMapLazy {
         simple.dist.get_or_init(get_or_init).as_ref()
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn source_dist_compatibility(
         &self,
         version: &Version,
         requires_python: Option<VersionSpecifiers>,
+        hashes: &[HashDigest],
         yanked: Option<Yanked>,
         excluded: bool,
         upload_time: Option<i64>,
@@ -462,14 +474,32 @@ impl VersionMapLazy {
             }
         }
 
-        SourceDistCompatibility::Compatible
+        // Check if hashes line up. If hashes aren't required, they're considered matching.
+        let hash = if self.required_hashes.is_empty() {
+            HashComparison::Matched
+        } else {
+            if hashes.is_empty() {
+                HashComparison::Missing
+            } else if hashes
+                .iter()
+                .any(|hash| self.required_hashes.contains(hash))
+            {
+                HashComparison::Matched
+            } else {
+                HashComparison::Mismatched
+            }
+        };
+
+        SourceDistCompatibility::Compatible(hash)
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn wheel_compatibility(
         &self,
         filename: &WheelFilename,
         version: &Version,
         requires_python: Option<VersionSpecifiers>,
+        hashes: &[HashDigest],
         yanked: Option<Yanked>,
         excluded: bool,
         upload_time: Option<i64>,
@@ -500,8 +530,31 @@ impl VersionMapLazy {
             }
         }
 
-        // Determine a compatibility for the wheel based on tags
-        WheelCompatibility::from(filename.compatibility(&self.tags))
+        // Determine a compatibility for the wheel based on tags.
+        let priority = match filename.compatibility(&self.tags) {
+            TagCompatibility::Incompatible(tag) => {
+                return WheelCompatibility::Incompatible(IncompatibleWheel::Tag(tag))
+            }
+            TagCompatibility::Compatible(priority) => priority,
+        };
+
+        // Check if hashes line up. If hashes aren't required, they're considered matching.
+        let hash = if self.required_hashes.is_empty() {
+            HashComparison::Matched
+        } else {
+            if hashes.is_empty() {
+                HashComparison::Missing
+            } else if hashes
+                .iter()
+                .any(|hash| self.required_hashes.contains(hash))
+            {
+                HashComparison::Matched
+            } else {
+                HashComparison::Mismatched
+            }
+        };
+
+        WheelCompatibility::Compatible(hash, priority)
     }
 }
 
