@@ -11,15 +11,19 @@ use chrono::{DateTime, Utc};
 use once_cell::sync::Lazy;
 
 use distribution_types::{IndexLocations, Requirement, Resolution, SourceDist};
-use pep508_rs::{MarkerEnvironment, StringVersion};
+use pep508_rs::{MarkerEnvironment, MarkerEnvironmentBuilder};
 use platform_tags::{Arch, Os, Platform, Tags};
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
-use uv_configuration::{BuildKind, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy};
-use uv_interpreter::{find_default_python, Interpreter, PythonEnvironment};
+use uv_configuration::{
+    BuildKind, Concurrency, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy,
+};
+use uv_distribution::DistributionDatabase;
+use uv_interpreter::{find_default_interpreter, Interpreter, PythonEnvironment};
 use uv_resolver::{
     DisplayResolutionGraph, ExcludeNewer, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options,
-    OptionsBuilder, PreReleaseMode, Preference, ResolutionGraph, ResolutionMode, Resolver,
+    OptionsBuilder, PreReleaseMode, Preference, PythonRequirement, ResolutionGraph, ResolutionMode,
+    Resolver,
 };
 use uv_types::{
     BuildContext, BuildIsolation, EmptyInstalledPackages, HashStrategy, SourceBuildTrait,
@@ -119,28 +123,33 @@ async fn resolve(
     markers: &'static MarkerEnvironment,
     tags: &Tags,
 ) -> Result<ResolutionGraph> {
-    let client = RegistryClientBuilder::new(Cache::temp()?).build();
+    let cache = Cache::temp().unwrap().init().unwrap();
+    let client = RegistryClientBuilder::new(cache).build();
     let flat_index = FlatIndex::default();
     let index = InMemoryIndex::default();
-    // TODO(konstin): Should we also use the bootstrapped pythons here?
-    let real_interpreter =
-        find_default_python(&Cache::temp().unwrap()).expect("Expected a python to be installed");
+    let real_interpreter = find_default_interpreter(&Cache::temp().unwrap())
+        .unwrap()
+        .expect("Python should be installed")
+        .into_interpreter();
     let interpreter = Interpreter::artificial(real_interpreter.platform().clone(), markers.clone());
-    let build_context = DummyContext::new(Cache::temp()?, interpreter.clone());
+    let python_requirement = PythonRequirement::from_marker_environment(&interpreter, markers);
+    let cache = Cache::temp().unwrap().init().unwrap();
+    let build_context = DummyContext::new(cache, interpreter.clone());
     let hashes = HashStrategy::None;
     let installed_packages = EmptyInstalledPackages;
+    let concurrency = Concurrency::default();
     let resolver = Resolver::new(
         manifest,
         options,
-        markers,
-        &interpreter,
+        &python_requirement,
+        Some(markers),
         tags,
-        &client,
         &flat_index,
         &index,
         &hashes,
         &build_context,
-        &installed_packages,
+        installed_packages,
+        DistributionDatabase::new(&client, &build_context, concurrency.downloads),
     )?;
     Ok(resolver.resolve().await?)
 }
@@ -722,19 +731,19 @@ async fn msgraph_sdk() -> Result<()> {
 }
 
 static MARKERS_311: Lazy<MarkerEnvironment> = Lazy::new(|| {
-    MarkerEnvironment {
-        implementation_name: "cpython".to_string(),
-        implementation_version: StringVersion::from_str("3.11.5").unwrap(),
-        os_name: "posix".to_string(),
-        platform_machine: "arm64".to_string(),
-        platform_python_implementation: "CPython".to_string(),
-        platform_release: "21.6.0".to_string(),
-        platform_system: "Darwin".to_string(),
-        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000".to_string(),
-        python_full_version: StringVersion::from_str("3.11.5").unwrap(),
-        python_version: StringVersion::from_str("3.11").unwrap(),
-        sys_platform: "darwin".to_string(),
-    }
+    MarkerEnvironment::try_from(MarkerEnvironmentBuilder {
+        implementation_name: "cpython",
+        implementation_version: "3.11.5",
+        os_name: "posix",
+        platform_machine: "arm64",
+        platform_python_implementation: "CPython",
+        platform_release: "21.6.0",
+        platform_system: "Darwin",
+        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000",
+        python_full_version: "3.11.5",
+        python_version: "3.11",
+        sys_platform: "darwin",
+    }).unwrap()
 });
 
 static TAGS_311: Lazy<Tags> = Lazy::new(|| {
@@ -755,19 +764,19 @@ static TAGS_311: Lazy<Tags> = Lazy::new(|| {
 });
 
 static MARKERS_310: Lazy<MarkerEnvironment> = Lazy::new(|| {
-    MarkerEnvironment {
-        implementation_name: "cpython".to_string(),
-        implementation_version: StringVersion::from_str("3.10.5").unwrap(),
-        os_name: "posix".to_string(),
-        platform_machine: "arm64".to_string(),
-        platform_python_implementation: "CPython".to_string(),
-        platform_release: "21.6.0".to_string(),
-        platform_system: "Darwin".to_string(),
-        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000".to_string(),
-        python_full_version: StringVersion::from_str("3.10.5").unwrap(),
-        python_version: StringVersion::from_str("3.10").unwrap(),
-        sys_platform: "darwin".to_string(),
-    }
+    MarkerEnvironment::try_from(MarkerEnvironmentBuilder {
+        implementation_name: "cpython",
+        implementation_version: "3.10.5",
+        os_name: "posix",
+        platform_machine: "arm64",
+        platform_python_implementation: "CPython",
+        platform_release: "21.6.0",
+        platform_system: "Darwin",
+        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000",
+        python_full_version: "3.10.5",
+        python_version: "3.10",
+        sys_platform: "darwin",
+    }).unwrap()
 });
 
 static TAGS_310: Lazy<Tags> = Lazy::new(|| {
