@@ -11,7 +11,7 @@ use platform_tags::Tags;
 use pypi_types::Yanked;
 use tracing::debug;
 use uv_cache::Cache;
-use uv_client::{BaseClientBuilder, RegistryClient, RegistryClientBuilder};
+use uv_client::{BaseClientBuilder, Connectivity, RegistryClient, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, ConfigSettings, Constraints, NoBinary, NoBuild, Overrides, PreviewMode, Reinstall,
     SetupPyStrategy,
@@ -20,10 +20,10 @@ use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
 use uv_installer::{Downloader, Plan, Planner, SatisfiesResult, SitePackages};
-use uv_interpreter::{find_default_python, Interpreter, PythonEnvironment};
+use uv_interpreter::{find_default_interpreter, Interpreter, PythonEnvironment};
 use uv_requirements::{
-    ExtrasSpecification, LookaheadResolver, NamedRequirementsResolver, RequirementsSource,
-    RequirementsSpecification, SourceTreeResolver,
+    ExtrasSpecification, LookaheadResolver, NamedRequirementsResolver, ProjectWorkspace,
+    RequirementsSource, RequirementsSpecification, SourceTreeResolver,
 };
 use uv_resolver::{
     Exclusions, FlatIndex, InMemoryIndex, Manifest, Options, OptionsBuilder, PythonRequirement,
@@ -31,13 +31,11 @@ use uv_resolver::{
 };
 use uv_types::{BuildIsolation, HashStrategy, InFlight, InstalledPackagesProvider};
 
-use crate::commands::project::discovery::Project;
 use crate::commands::reporters::{DownloadReporter, InstallReporter, ResolverReporter};
 use crate::commands::{elapsed, ChangeEvent, ChangeEventKind};
 use crate::editables::ResolvedEditables;
 use crate::printer::Printer;
 
-mod discovery;
 pub(crate) mod lock;
 pub(crate) mod run;
 pub(crate) mod sync;
@@ -80,19 +78,22 @@ pub(crate) enum Error {
 
 /// Initialize a virtual environment for the current project.
 pub(crate) fn init(
-    project: &Project,
+    project: &ProjectWorkspace,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment, Error> {
-    let venv = project.root().join(".venv");
+    let venv = project.workspace().root().join(".venv");
 
     // Discover or create the virtual environment.
     // TODO(charlie): If the environment isn't compatible with `--python`, recreate it.
     match PythonEnvironment::from_root(&venv, cache) {
         Ok(venv) => Ok(venv),
-        Err(uv_interpreter::Error::VenvDoesNotExist(_)) => {
+        Err(uv_interpreter::Error::NotFound(_)) => {
             // TODO(charlie): Respect `--python`; if unset, respect `Requires-Python`.
-            let interpreter = find_default_python(cache)?;
+            let interpreter = find_default_interpreter(cache)
+                .map_err(uv_interpreter::Error::from)?
+                .map_err(uv_interpreter::Error::from)?
+                .into_interpreter();
 
             writeln!(
                 printer.stderr(),
@@ -463,15 +464,16 @@ pub(crate) async fn install(
 }
 
 /// Update a [`PythonEnvironment`] to satisfy a set of [`RequirementsSource`]s.
-async fn update_environment(
+pub(crate) async fn update_environment(
     venv: PythonEnvironment,
     requirements: &[RequirementsSource],
     preview: PreviewMode,
+    connectivity: Connectivity,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment> {
     // TODO(zanieb): Support client configuration
-    let client_builder = BaseClientBuilder::default();
+    let client_builder = BaseClientBuilder::default().connectivity(connectivity);
 
     // Read all requirements from the provided sources.
     // TODO(zanieb): Consider allowing constraints and extras
@@ -523,6 +525,7 @@ async fn update_environment(
     // Initialize the registry client.
     // TODO(zanieb): Support client options e.g. offline, tls, etc.
     let client = RegistryClientBuilder::new(cache.clone())
+        .connectivity(connectivity)
         .markers(markers)
         .platform(venv.interpreter().platform())
         .build();
@@ -571,7 +574,10 @@ async fn update_environment(
     // Build all editable distributions. The editables are shared between resolution and
     // installation, and should live for the duration of the command.
     let editables = ResolvedEditables::resolve(
-        spec.editables.clone(),
+        spec.editables
+            .iter()
+            .cloned()
+            .map(ResolvedEditables::from_requirement),
         &site_packages,
         &reinstall,
         &hasher,
