@@ -15,11 +15,13 @@ use install_wheel_rs::linker::LinkMode;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{Connectivity, FlatIndexClient, RegistryClientBuilder};
-use uv_configuration::KeyringProviderType;
+use uv_configuration::{Concurrency, KeyringProviderType};
 use uv_configuration::{ConfigSettings, IndexStrategy, NoBinary, NoBuild, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
-use uv_interpreter::{find_default_python, find_requested_python, Error};
+use uv_interpreter::{
+    find_default_interpreter, find_interpreter, InterpreterRequest, SourceSelector,
+};
 use uv_resolver::{ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder};
 use uv_types::{BuildContext, BuildIsolation, HashStrategy, InFlight};
 
@@ -116,14 +118,17 @@ async fn venv_impl(
     printer: Printer,
 ) -> miette::Result<ExitStatus> {
     // Locate the Python interpreter.
-    let interpreter = if let Some(python_request) = python_request {
-        find_requested_python(python_request, cache)
-            .into_diagnostic()?
-            .ok_or(Error::NoSuchPython(python_request.to_string()))
-            .into_diagnostic()?
+    let interpreter = if let Some(python) = python_request.as_ref() {
+        let system = uv_interpreter::SystemPython::Required;
+        let request = InterpreterRequest::parse(python);
+        let sources = SourceSelector::from_settings(system);
+        find_interpreter(&request, system, &sources, cache)
     } else {
-        find_default_python(cache).into_diagnostic()?
-    };
+        find_default_interpreter(cache)
+    }
+    .into_diagnostic()?
+    .into_diagnostic()?
+    .into_interpreter();
 
     // Add all authenticated sources to the cache.
     for url in index_locations.urls() {
@@ -194,8 +199,9 @@ async fn venv_impl(
         // Track in-flight downloads, builds, etc., across resolutions.
         let in_flight = InFlight::default();
 
-        // For seed packages, assume the default settings are sufficient.
+        // For seed packages, assume the default settings and concurrency is sufficient.
         let config_settings = ConfigSettings::default();
+        let concurrency = Concurrency::default();
 
         // Prep the build context.
         let build_dispatch = BuildDispatch::new(
@@ -212,6 +218,7 @@ async fn venv_impl(
             link_mode,
             &NoBuild::All,
             &NoBinary::None,
+            concurrency,
         )
         .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
 
