@@ -10,6 +10,7 @@ use clap::{CommandFactory, Parser};
 use owo_colors::OwoColorize;
 use tracing::instrument;
 
+use cli::{ToolCommand, ToolNamespace};
 use uv_cache::Cache;
 use uv_requirements::RequirementsSource;
 use uv_workspace::Combine;
@@ -43,6 +44,7 @@ static GLOBAL: tikv_jemallocator::Jemalloc = tikv_jemallocator::Jemalloc;
 mod cli;
 mod commands;
 mod compat;
+mod editables;
 mod logging;
 mod printer;
 mod settings;
@@ -112,7 +114,7 @@ async fn run() -> Result<ExitStatus> {
     // 3. The user configuration file.
     let workspace = if let Some(config_file) = cli.config_file.as_ref() {
         Some(uv_workspace::Workspace::from_file(config_file)?)
-    } else if cli.isolated {
+    } else if cli.global_args.isolated {
         None
     } else {
         let project = uv_workspace::Workspace::find(env::current_dir()?)?;
@@ -176,8 +178,14 @@ async fn run() -> Result<ExitStatus> {
 
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipCompileSettings::resolve(args, workspace);
+            rayon::ThreadPoolBuilder::new()
+                .num_threads(args.shared.concurrency.installs)
+                .build_global()
+                .expect("failed to initialize global rayon pool");
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
 
-            let cache = cache.with_refresh(args.refresh);
             let requirements = args
                 .src_file
                 .into_iter()
@@ -219,7 +227,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.keyring_provider,
                 args.shared.setup_py,
                 args.shared.config_setting,
-                args.shared.connectivity,
+                globals.connectivity,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
                 args.shared.python_version,
@@ -229,6 +237,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.link_mode,
                 args.shared.python,
                 args.shared.system,
+                args.shared.concurrency,
                 args.uv_lock,
                 globals.native_tls,
                 globals.quiet,
@@ -245,16 +254,28 @@ async fn run() -> Result<ExitStatus> {
 
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipSyncSettings::resolve(args, workspace);
+            rayon::ThreadPoolBuilder::new()
+                .num_threads(args.shared.concurrency.installs)
+                .build_global()
+                .expect("failed to initialize global rayon pool");
 
-            let cache = cache.with_refresh(args.refresh);
-            let sources = args
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            let requirements = args
                 .src_file
                 .into_iter()
                 .map(RequirementsSource::from_requirements_file)
                 .collect::<Vec<_>>();
+            let constraints = args
+                .constraint
+                .into_iter()
+                .map(RequirementsSource::from_constraints_txt)
+                .collect::<Vec<_>>();
 
             commands::pip_sync(
-                &sources,
+                &requirements,
+                &constraints,
                 &args.reinstall,
                 args.shared.link_mode,
                 args.shared.compile_bytecode,
@@ -263,7 +284,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.index_strategy,
                 args.shared.keyring_provider,
                 args.shared.setup_py,
-                args.shared.connectivity,
+                globals.connectivity,
                 &args.shared.config_setting,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
@@ -271,13 +292,16 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python_version,
                 args.shared.python_platform,
                 args.shared.strict,
+                args.shared.exclude_newer,
                 args.shared.python,
                 args.shared.system,
                 args.shared.break_system_packages,
                 args.shared.target,
+                args.shared.concurrency,
                 globals.native_tls,
                 globals.preview,
                 cache,
+                args.dry_run,
                 printer,
             )
             .await
@@ -286,10 +310,16 @@ async fn run() -> Result<ExitStatus> {
             command: PipCommand::Install(args),
         }) => {
             args.compat_args.validate()?;
+
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipInstallSettings::resolve(args, workspace);
+            rayon::ThreadPoolBuilder::new()
+                .num_threads(args.shared.concurrency.installs)
+                .build_global()
+                .expect("failed to initialize global rayon pool");
 
-            let cache = cache.with_refresh(args.refresh);
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
             let requirements = args
                 .package
                 .into_iter()
@@ -329,7 +359,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.compile_bytecode,
                 args.shared.require_hashes,
                 args.shared.setup_py,
-                args.shared.connectivity,
+                globals.connectivity,
                 &args.shared.config_setting,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
@@ -342,6 +372,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.system,
                 args.shared.break_system_packages,
                 args.shared.target,
+                args.shared.concurrency,
                 args.uv_lock,
                 globals.native_tls,
                 globals.preview,
@@ -357,6 +388,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipUninstallSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             let sources = args
                 .package
                 .into_iter()
@@ -374,7 +408,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.break_system_packages,
                 args.shared.target,
                 cache,
-                args.shared.connectivity,
+                globals.connectivity,
                 globals.native_tls,
                 globals.preview,
                 args.shared.keyring_provider,
@@ -388,6 +422,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipFreezeSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::pip_freeze(
                 args.exclude_editable,
                 args.shared.strict,
@@ -405,6 +442,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipListSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::pip_list(
                 args.editable,
                 args.exclude_editable,
@@ -423,6 +463,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipShowSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::pip_show(
                 args.package,
                 args.shared.strict,
@@ -438,6 +481,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipCheckSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::pip_check(
                 args.shared.python.as_deref(),
                 args.shared.system,
@@ -464,6 +510,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = settings::VenvSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             // Since we use ".venv" as the default name, we use "." as the default prompt.
             let prompt = args.prompt.or_else(|| {
                 if args.name == PathBuf::from(".venv") {
@@ -482,7 +531,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.keyring_provider,
                 uv_virtualenv::Prompt::from_args(prompt),
                 args.system_site_packages,
-                args.shared.connectivity,
+                globals.connectivity,
                 args.seed,
                 args.allow_existing,
                 args.shared.exclude_newer,
@@ -496,6 +545,9 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = settings::RunSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             let requirements = args
                 .with
                 .into_iter()
@@ -521,8 +573,9 @@ async fn run() -> Result<ExitStatus> {
                 args.args,
                 requirements,
                 args.python,
-                cli.isolated,
+                globals.isolated,
                 globals.preview,
+                globals.connectivity,
                 &cache,
                 printer,
             )
@@ -532,12 +585,18 @@ async fn run() -> Result<ExitStatus> {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let _args = settings::SyncSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::sync(globals.preview, &cache, printer).await
         }
         Commands::Lock(args) => {
             // Resolve the settings from the command-line arguments and workspace configuration.
             let _args = settings::LockSettings::resolve(args, workspace);
 
+            // Initialize the cache.
+            let cache = cache.init()?;
+
             commands::lock(globals.preview, &cache, printer).await
         }
         #[cfg(feature = "self-update")]
@@ -552,6 +611,21 @@ async fn run() -> Result<ExitStatus> {
             shell.generate(&mut Cli::command(), &mut stdout());
             Ok(ExitStatus::Success)
         }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::Run(args),
+        }) => {
+            commands::run_tool(
+                args.target,
+                args.args,
+                args.python,
+                globals.isolated,
+                globals.preview,
+                globals.connectivity,
+                &cache,
+                printer,
+            )
+            .await
+        }
     }
 }
 
